/*
 * OutputWriter.h
 *
 * Copyright Notice: see copyright.txt
 *
 * Date: 3/8/2012 (D/M/Y)
 * Author: Dmitrij Lioubartsev
 * Email: dmitrijl42@gmail.com
 *
 * This is a helper class for creating files of different formats.
 * Can create .xmol, .smol and .car .
 */

#ifndef OUTPUTWRITER_H_
#define OUTPUTWRITER_H_

#include <string>
using namespace std;

#include "CifCrystal.h"
#include "ForceField.h"
#include "ForceFieldJoint.h"

class OutputWriter {
private:
	string output;

public:
	OutputWriter(string outputFileName);
	virtual ~OutputWriter();

	void createXmol(CifCrystal& crystal); //creating an xmol-file
	//void createSmolDebug(CifCrystal& crystal); //creating a simple smol file for testing purposes
	void createSmol(CifCrystal& crystal); //Create the final smol file
	void printCrystal(CifCrystal& crystal); //print crystal info into stdout, for testing purposes.
	void createCar(CifCrystal& crystal, string title); //create a car file (you wouldn't download a car would you?)

private:
	void removeFileExtension(string& fileName); //removes the file extension, if there is any.
	void setPrecision(string& doubl, int precision = 4); //keeps only precision number of numbers
														//after decimal point.
	void writeAndFill(string writeThis, int space, ofstream& stream);
};

#endif /* OUTPUTWRITER_H_ */
