/*
 * SymmetryCalculator.h
 *
 * Copyright Notice: see copyright.txt
 *
 * Date: 3/8/2012 (D/M/Y)
 * Author: Dmitrij Lioubartsev
 * Email: dmitrijl42@gmail.com
 *
 * A symmetry is a string that holds mathematic operations for
 * three coordinates separated by commas. Eg. "x+1/2,y+1/2,z+1/2".
 * x,y,z are the current atom's coordinates. The three expressions give
 * a new value for each coordinate. This class calculates each expression.
 *
 * Esentially all common expressions using +,-,*,/,(,) are allowed. Syntactic
 * sugar as '3x' or 'x(1+2)4' is allowed as well. The parsing process will simply add *
 * to compensate.
 *
 * How to use this class:
 *
 * 1. Use the constructor (does nothing)
 * 2. setSymmetry(Symmetry string)
 * 			This loads and parses the symmetry string
 * 3. loadAtom - input atom coordinates
 * 4. call calculate();
 * 5. Use getx(), gety(), getz() to fetch calculated values.
 *
 */

#ifndef SYMMETRYCALCULATOR_H_
#define SYMMETRYCALCULATOR_H_

#include <string>
using namespace std;

class SymmetryCalculator {
private:
	string xyz[3]; //the strings of the symmetry
	double xyzNVal[3]; //the new atom values, after symmetry is applied.
	double xyzOVal[3]; //the old atom values, before symmetry is applied.
	struct res {
		double result;
		int ip;
	};

public:
	SymmetryCalculator();
	virtual ~SymmetryCalculator();

	bool setSymmetry(const string& symmetry);
	void loadAtom(double x, double y, double z);
	bool calculate();

	//getters
	double getx();
	double gety();
	double getz();

private:
	//here follows the private intern calculation methods.
	//this calculates all terms (+ and -)
	res calculateTerms(string& expr, int expp);
	//this calculates all products (* and /)
	res calculateProducts(string& expr, int expp);
	//this fetches a value. If its a parenthesis expression, uses recursion
	res getValue(string& expr, int expp);

};

#endif /* SYMMETRYCALCULATOR_H_ */
